/*
 * ===========================================================================
 *
 * $Id: adminlib.inc,v 1.5 2003/07/27 20:16:05 darope Exp $
 *
 *
 * Copyright (c) 2001-2003 Alfred Reynolds, Florian Zschocke, Magua
 *
 *   This file is part of Admin Mod.
 *
 *   Admin Mod is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Admin Mod is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Admin Mod; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   In addition, as a special exception, the author gives permission to
 *   link the code of this program with the Half-Life Game Engine ("HL 
 *   Engine") and Modified Game Libraries ("MODs") developed by VALVe, 
 *   L.L.C ("Valve") and Modified Game Libraries developed by Gearbox 
 *   Software ("Gearbox").  You must obey the GNU General Public License 
 *   in all respects for all of the code used other than the HL Engine and 
 *   MODs from Valve or Gearbox. If you modify this file, you may extend 
 *   this exception to your version of the file, but you are not obligated 
 *   to do so.  If you do not wish to do so, delete this exception statement
 *   from your version.
 *
 * ===========================================================================
 *
 * Comments:
 *
 *   adminmod helper function library
 *
 * added by Sir D.:
 * - allowsound(User[])
 * - log_in_file(sCommand[],sData[],iFile=2)
 * - numtoword(iNum,str[],iLength)
 * - get_access(UserIndex)
 * - isip(IP[])
 * - issteamid(SteamID[])
 * - strstrip(str[],chars)
 * - streqindex(str[],test[],pos)
 * - strsplitx(str[],separator,escape,...)
 * - strfilter(str[],allow[])
 * - strseqlen(str[],allow[])
 * - close_menu(User[],UserIndex)
 * - changed strbreak using strgsep
 * - sound_to_all(Data[],iTeam=0){
 */
#if defined _adminlib_included
  #endinput
#endif
#define _adminlib_included

#include <admin>
#include <core>
#include <string>

#define CHAR_INVALID -1
#define FALSE 0
#define NULL_CHAR 0
#define PERIOD_CHAR 46
#define QUOTE_CHAR 34
#define TRUE 1



/* Returns 1 if the current user has the required auth level, 0 otherwise */
stock check_auth( iAuthLevel ) {
	new iResult = access(iAuthLevel,"");
	if (iResult!=0) iResult = 1;
	return iResult;
}

/* Returns 1 if the target has immunity */
stock check_immunity( sTarget[] ) {
	/* If the server doesn't honor immunity, no one is immune. */
	if (getvar("admin_ignore_immunity")!=0) {
		return 0;
	/* Else, you're immune if you have that access level. */
	} else if (access(ACCESS_IMMUNITY,sTarget)!=0) {
		return 1;
	}
	return 0;
}

/* I had this in the normal alphabetical order...but it caused
the Small compiler to barf.  So I moved it down here, and it's
fine.  That doesn't fill me with inspiration, but it seems
to work. */
/* Returns 1 if param is '1' or 'on'...0 for anything else. */
stock check_param( sParam[] ) {
	new iResult = 0;
	
	if(strlen(sParam) > 0) {
		if (streq(sParam, "on")==1) {
			iResult = 1;
		} else if (streq(sParam, "1")==1) {
			iResult = 1;
		}
	}
	return iResult;
}

////////////////////
// MENU FUNCTIONS //
////////////////////

//send close to all plugins
stock close_menu(UserIndex){
	if(1<= UserIndex <=MAX_PLAYERS){
		new UserName[MAX_NAME_LENGTH];
		new sUserIndex[MAX_NUMBER_LENGTH];
		if(playerinfo(UserIndex,UserName,MAX_NAME_LENGTH)){
			snprintf(sUserIndex,MAX_NUMBER_LENGTH,"%i",UserIndex);
			plugin_exec("close_menu",sUserIndex);
		}
	}
}

//set_vaultnum, that user is in adminmod-menu
stock open_menu(UserIndex){
	if(1<= UserIndex <=MAX_PLAYERS){
		new Text[MAX_TEXT_LENGTH];
		snprintf(Text,MAX_TEXT_LENGTH,"menu_%i",UserIndex);
		set_vaultnumdata(Text,1);
	}
}

//set_vaultnum, that user has left adminmod-menu
stock reset_menu(UserIndex){
	if(1<= UserIndex <=MAX_PLAYERS){
		if(check_menu_open(UserIndex)){
			new Text[MAX_TEXT_LENGTH];
			snprintf(Text,MAX_TEXT_LENGTH,"menu_%i",UserIndex);
			set_vaultdata(Text,"");
		}
	}
}

//check vault if user is still in menu or has left
stock check_menu_open(UserIndex){
	if(1<= UserIndex <=MAX_PLAYERS){
		new Text[MAX_TEXT_LENGTH];
		new i;
		snprintf(Text,MAX_TEXT_LENGTH,"menu_%i",UserIndex);
		get_vaultnumdata(Text,i);
		return i;
	}
	return 0;
}

stock convert_string_all(HLCommand,HLData,HLUserName,Command[],Data[],User[]){
	convert_string(HLCommand,Command,MAX_COMMAND_LENGTH);
	convert_string(HLData,Data,MAX_DATA_LENGTH);
	convert_string(HLUserName,User,MAX_NAME_LENGTH);
}

stock execclient_all( sCommand[] ) {
	new Target[MAX_NAME_LENGTH];
	new i;
	new iSessionID;
	new sSessionID[MAX_NUMBER_LENGTH];
	new maxplayers = maxplayercount();

	for(i=1; i<=maxplayers; i++) {
		strinit(Target);
		if(playerinfo(i,Target,MAX_NAME_LENGTH, iSessionID)==1) {
			numtostr( iSessionID, sSessionID );
			execclient(sSessionID,sCommand);

		}  // if
	}  // for

	return 1;
}  // execclient_all()

/* Executes a command while providing the nicely formatted output */
stock execute_command(sUser[], sCommand[], sHalfLifeCmd[], sData[]) {
	new sRconCmd[MAX_DATA_LENGTH];
	
	say_command(sUser,sCommand,sData);
	snprintf(sRconCmd, MAX_DATA_LENGTH, "%s %s", sHalfLifeCmd, sData);
	exec(sRconCmd);
}

/* Nicely formats the current command */
stock format_command(sUser[],sCommand[],sData[],sText[]) {
	snprintf(sText, MAX_TEXT_LENGTH, "Command: %s used command %s %s", sUser, sCommand, sData);
}

/* Logs a command */
stock log_command(sUser[],sCommand[],sData[]) {
	new sText[MAX_TEXT_LENGTH];
	format_command(sUser,sCommand,sData,sText);
	log(sText);
}

stock get_access(UserIndex){
	new Total = 0;
	new User[MAX_NAME_LENGTH];
	
	if (playerinfo(UserIndex,User,MAX_NAME_LENGTH) ){
		for(new i=1;i<=262144;i += i){
			if (access(i,User)!=0){
				Total += i;
			}
		}
	} else {
		return -1;
	}
	return Total;
}

stock get_username_by_sid(iSessionID,User[]){
	new iPLSessionID;
	new iUserIndex;
	new i;
	new maxplayers=maxplayercount();
	for(i=1;i<=maxplayers;i++){
		if(playerinfo(i,User,MAX_NAME_LENGTH,iPLSessionID)){
			if(iSessionID==iPLSessionID){
				iUserIndex=i;
				break;
			}
		}
	}
	return iUserIndex;
}

stock isip(IP[]){
	new Flag=0;
	new Dots=0;
	for(new i=0;IP[i]!=0;i++){
		if ( !((IP[i] <= '9' && IP[i] >= '0') || IP[i]=='.' ) ){
			Flag = 1;
		} else if (IP[i]=='.'){
			Dots++;
		}
	}
	if (Dots==3){
		return Flag;
	}
	return 0;
}

stock issteamid(SteamID[]){
	if (strncasecmp(SteamID,"Steam_",6) == 0){
		return 1;
	}
	return 0;
}

/* Log commands in file*/
stock log_to_file(UserIndex,Command[],Data[],File[]){
	new iwrite;
	iwrite = getvar("file_access_write");
	if(iwrite){
		new Text[MAX_TEXT_LENGTH];
		new ST[MAX_DATA_LENGTH];
		new User[MAX_NAME_LENGTH];
		new uAuthID[MAX_AUTHID_LENGTH];
		new uiSession;
		new uiTeam;

		if(UserIndex!=0){
			playerinfo(UserIndex,User,MAX_NAME_LENGTH,uiSession,_,uiTeam,_,uAuthID);
		}else{
			strcpy(User,"RCON/ADMIN",MAX_NAME_LENGTH);
			strcpy(uAuthID,"-1",MAX_AUTHID_LENGTH);
			uiTeam=-1;
		}
		servertime(ST, MAX_DATA_LENGTH, "L %m/%d/%y - %H:%M:%S:");

		//write data to global log files
		snprintf(Text,MAX_TEXT_LENGTH,"%s ^"%s<%i><%s><%i>^" %s ^"%s^"",ST,User,uiSession,uAuthID,uiTeam,Command,Data);
		writefile(File,Text,-1);
		return 1;
	}
	return 0;
}

/* Returns the lesser of a and b */
stock min(a,b) {
	if (a<b) return a;
	else return b;
	return 1;
}

/* Returns the greater of a and b */
stock max(a,b) {
	if (a>b) return a;
	else return b;
	return 1;
}

/* NumToStr courtesy of Nathan O'Sullivan (http://nathan.qgl.org/halflifeadmin/numtostr.txt) */
stock numtostr(num,str[]) {
	new Base = 1;
	new Digits = 1;
	new i = 0;
	
	/* Special case: 0 */
	if (num == 0) {
		str[i++] = '0';
		str[i++] = NULL_CHAR;
	} else {
		/* If we've got a negative number, add a negative sign
		to the string, and multiply the number by -1 */
		if (num < 0) {
			str[i++] = '-';	
			num *= -1;
		}
	
		/* Ok.  We've got at least one digit.  Keep multiplying by
		10 till we get a higher number than what we've got.  Note 
		that this will leave Digits 1 higher than what we want (eg,
		if Num was 7, Digits will be 2 */
		while (Base <= num) {
			Base *= 10;
			Digits++;
		}
		
		/* Because Digits is higher, use --Digits rather than Digits--. 
		Also, divide Base before using it. */
		while (--Digits > 0) {
			Base /= 10;
			str[i++] = '0' + (num - (num % Base)) / Base;
			num = num % Base;
		}
		str[i++] = NULL_CHAR;
	}
}

/* numtoword function
   converts an integer number (e.g. 12) to
   a text (e.g. "twelve"). works for all
   numbers between 0 and 999,999,999,999,999
 */

static OnesLib[10][] = { "", "one ", "two ", "three ", "four ", "five ", "six ", "seven ", "eight ", "nine " };
static TensLib[10][] = {"", "ten ", "twenty ", "thirty ", "fourty ", "fifty ", "sixty ", "seventy ", "eighty ", "ninety " } ;
static TeensLib[10][] = {"", "eleven ", "twelve ", "thirteen ", "fourteen ", "fifteen ", "sixteen ", "seventeen ", "eighteen ", "nineteen " };
static HundredsLib[10][] = { "", "one hundred ", "two hundred ", "three hundred ", "four hundred ", "five hundred ", "six hundred ", "seven hundred ", "eight hundred ", "nine hundred " };
static GroupNamesLib[5][] = {"", "thousand ", "million ", "billion ", "trillion " };

stock numtoword(iNum,str[],iLength) {
   /* Empties the first char of the string */
   str[0] = 0;
   str[iLength-1] = 0;
   if(iNum == 0) {
      snprintf(str,iLength,"zero");
      return 1;
   }
   if(iNum < 0) {
      iNum *= -1;
      snprintf(str,iLength,"negative ");
   }

   return do_numtoword(iNum,str,0,iLength);
}

static stock do_numtoword(iNum,str[],iLevel,iLength) {

   /* If there is more than one group of 3, then take care of the others first. */
   if(iNum >= 1000) do_numtoword(iNum / 1000,str,iLevel + 1,iLength);

   /* Grab the the first 3 digits only */
   iNum %= 1000;

   /* Isolate each digit */
   new _hundreds = iNum / 100;
   new _tens = (iNum / 10) % 10;
   new _ones = iNum % 10;

   /* Take care of the teen numbers */
   if(_tens == 1 && _ones != 0) {
      snprintf(str,iLength,"%s%s%s%s",str,HundredsLib[_hundreds],TeensLib[_ones],GroupNamesLib[iLevel]);
   } else {
      snprintf(str,iLength,"%s%s%s%s%s",str,HundredsLib[_hundreds], TensLib[_tens], OnesLib[_ones],GroupNamesLib[iLevel]);
   }

   if(iLevel==0) str[strlen(str)-1] = 0; /* Gets rid of the trailing space*/
   return 1;
}

/* Command to use when the person doesn't have the proper permissions. */
stock reject_message(iPublic = 0) {
	new sText[MAX_TEXT_LENGTH];
	
	getstrvar("admin_reject_msg",sText,MAX_TEXT_LENGTH);
	if(strlen(sText) < 2) 
		strcpy(sText, "You do not have access to this command.", MAX_TEXT_LENGTH);
	if (iPublic == 0) {
		selfmessage(sText);
	} else {
		say(sText);
	}
}

/* Results differ upon the setting of admin_quiet.  If admin_quiet
is 0, says who executed what command.  If admin_quiet is 1, says
'Admin' executed what command. Otherwise, simply logs. 
The override option allows one to ignore admin_quiet, and always
display a message.
*/
stock say_command(sUser[],sCommand[],sData[], iOverride = 0) {
	new iQuiet;
	new sText[MAX_TEXT_LENGTH];
	
	iQuiet = getvar("admin_quiet");
	if (iQuiet==0) {
		format_command(sUser,sCommand,sData,sText);
		say(sText);
	} else if (iQuiet==1 || iOverride==1) {
		format_command("Admin",sCommand,sData,sText);
		say(sText);
	} else {
		log_command(sUser,sCommand,sData);
	}
}

new g_SetInfoKey[]="am_sound";

stock sound_get_key(User[]){
	new Info[MAX_NUMBER_LENGTH];
	new iInfo;
	new Authid[MAX_AUTHID_LENGTH];
	get_userAuthID(User,Authid,MAX_AUTHID_LENGTH);
	if(strcmp(Authid,"BOT")!=0){
		get_userinfo(User,g_SetInfoKey,Info,MAX_NUMBER_LENGTH);
		iInfo=strtonum(Info);
		if(strlen(Info)==0 || iInfo >=1){/*no info, sound is enabled till player use ".stop"*/
			return 1;
		}
	}
	return 0;
}

stock sound_set_key(User[],iValue){
	new SetInfo[MAX_TEXT_LENGTH];
	snprintf(SetInfo,MAX_TEXT_LENGTH,"setinfo ^"%s^" ^"%i^"",g_SetInfoKey,iValue);
	execclient(User,SetInfo);
}

stock sound_to_all(Data[],iTeam=0){
	new Player[MAX_NAME_LENGTH];
	new i;
	new maxplayers=maxplayercount();
	new iPlayerTeam;
	
	for(i=1;i<=maxplayers;i++){
		if(playerinfo(i,Player,MAX_NAME_LENGTH,_,_,iPlayerTeam)){
			if(sound_get_key(Player)){
				if(!iTeam){
					speakto(Player,Data);
				}else if (iTeam==iPlayerTeam){
					speakto(Player,Data);
				}
			}
		}
	}
}

/* new strbreak function*/
stock strbreak(str[], first[], second[], maxlen=0, flen=sizeof first, slen=sizeof second ) {
	if (maxlen == 0){
		maxlen = strlen(str);
	}
	strgsep(str," ","^"",first,flen,second,slen);
}

/* Given a string (str[]), this will attempt to break it apart at the first 
space that's not inside quotation marks.  Quotes at the beginning and end
will be stripped.  Ie,

new str[20] = "This is a test"
new first[20];
new second[20];
strbreak(str,first,second, 20);

first is now equal to "This", and second is now equal to "is a test".  If 
str[] had been "^"This is^" a test", first would be "This is" and second would 
be "a test". 
At maximum maxlen characters will be copied. */
/*
stock strbreak(str[], first[], second[], maxlen, flen=sizeof first, slen=sizeof second ) {
	new i = 0;
	new j = 0;
	new NullPos = CHAR_INVALID;
	new Quote = FALSE;
	new SpacePos = CHAR_INVALID;
	new iFirstEnd;
	new iSecondEnd;

	if (maxlen == 0)
		maxlen = strlen(str);



	Find the position of the first unquoted space and the terminating NULL character
	for(i=0; i<=strlen(str); i++) {
		if (str[i] == QUOTE_CHAR) {
			if (Quote==FALSE) {
				Quote = TRUE;
			} else {
				Quote = FALSE;
			}
		} else if (str[i] == ' ' && SpacePos == CHAR_INVALID && Quote == FALSE) {
			SpacePos = i;
		} else if (str[i] == NULL_CHAR) {
			NullPos = i;
			break;
		}	
	}
	
	if (NullPos == CHAR_INVALID) NullPos = maxlen + 1;


	if (SpacePos == CHAR_INVALID) {
		strcpy(first, str, flen);
		strinit(second);

	} else {		
		Make sure we do not copy more characters into the second array
		than would fit into it. If the second part of the source string
		is longer than the length of the second array, truncate it to fit.
		new iSecondLen = NullPos - SpacePos -1;
		iSecondEnd = min( iSecondLen, (slen-1) );

		If maxlen is set we want to copy only max maxlen characters.
		Check if this is smaller than our current limit
		iSecondEnd = min( iSecondEnd, maxlen );

		Make sure we do not copy more characters into the first array
		than would fit into it. If the first part of the source string
		is longer than the length of the first array, truncate it to fit.
		iFirstEnd = min( SpacePos, (flen-1) );

		If maxlen is set we want to copy only max maxlen characters.
		Check if this is smaller than our current limit
		iFirstEnd = min( iFirstEnd, maxlen );

	
		Copy the first part of the string into the first array
		for( i = 0; i < iFirstEnd; i++)  {
			first[i] = str[i];
		}
		Zero-terminate the first array
		first[iFirstEnd] = NULL_CHAR;
		

		Copy the second part of the string into the second array
		for( i = SpacePos+1, j = 0; j < iSecondEnd; i++, j++) {
			second[j] = str[i];
		}
		Zero-terminate the second array
		second[iSecondEnd] = NULL_CHAR;
	}

	
	Strip out the quotes of the return values.
	strstripquotes(first);
	strstripquotes(second);
}*/

/* Returns 1 if two strings are exactly the same, including length, etc */
stock streq( strOne[], strTwo[] ) {
	if (strlen(strOne) != strlen(strTwo)) {
		return 0;
	} else if (strcasecmp(strOne, strTwo)==0) {
		return 1;
	}
	return 0;
}

/* Initialize a string */
stock strinit( sString[] ) {
	sString[0] = NULL_CHAR;
}

/* returns 1 if two strings are the same in the first Length (or less 
if one is smaller than Length) digits, 0 otherwise */
stock strmatch( sOne[], sTwo[], iLength) {
	if (strncasecmp(sOne, sTwo, iLength)==0) {
		return 1;
	}
	return 0;
}

/* Strips the quotes from the beginning and ending, if they exist. Ignores
those in the middle. */
stock strstripquotes(str[]) {
	new maxlen = strlen(str);
	new i;
	
	if(maxlen==0)
		return;
		
	if(str[maxlen - 1]==QUOTE_CHAR) 
		str[--maxlen] = NULL_CHAR;
		
	if(str[0] == QUOTE_CHAR) {
		for(i=0; i<=maxlen; i++)
			str[i] = str[i+1];
		str[i-2] = NULL_CHAR;
	}
}


/* Removes the first l characters from str */
/*
stock strstrip(str[],chars, maxlen = sizeof str) {
    strcpy(str,str[chars],maxlen-chars);
}*/

stock strstrip(str[],chars) {
  new i=chars;
  new l=strlen(str);
  if (i>l) {
    str[0]=0;
  }
  else {
    while(i<=l) {
      str[i-chars]=str[i];
      i++;
    }
  }
}

/* Like strsplit, but spliting must be done on a single character instead
 * of a string, and that character can be escaped with another one */
stock strsplitx(str[],separator,escape,...) {

  new pos=0;
  new len=strlen(str);
  new escaped=0;
  new splits=numargs();
  new split=3;
  new splitpos=0;
  new splitlen=0;

  while (split +3 < splits) {
    splitpos=0;
    splitlen= getarg(split+1);

    /* Copy characters from str into the split argument while there is room */
    while ((pos<len) && ((str[pos] != separator) || (escaped==1)) && (splitpos < splitlen-1)) {
      if ((str[pos] == escape) && (escaped==0)) {
        escaped=1;
      }
      else {
        setarg(split,splitpos,str[pos]);
        splitpos++;
        escaped=0;
      }
      pos++;
    }
    setarg(split,splitpos,NULL_CHAR);

    /* If the split output buffer is too small, throw away characters */
    while ((pos<len) && ((str[pos] != separator) || (escaped==1))) {
      if ((str[pos] == escape) && (escaped==0)) {
        escaped=1;
      }
      else {
        escaped=0;
      }
      pos++;
    }

    /* Move to next output argument */
    split=split+2;
    pos++;
  }

  /* Copy remaining characters from the split */
  splitpos=0;
  splitlen= getarg(split+1);

  while ((pos<len) && (splitpos < splitlen-1)) {
    if ((str[pos] == escape) && (escaped==0)) {
      escaped=1;
    }
    else {
      setarg(split,splitpos,str[pos]);
      splitpos++;
      escaped=0;
    }
    pos++;
  }
  setarg(split,splitpos,NULL_CHAR);

  return (split-1)/2+1;
}



/* Removes all characters from str that don't appear in allow */
stock strfilter(str[],allow[]) {
  new c = strlen(str);
  new i = 0;
  new j = 0;
  new p;

  while (i<c) {
    p = index(allow,str[i]);
    if (p>=0) {
      str[j]=str[i];
      j++;
    }
    i++;
  }
  str[j]=0;

  return (i=j);
}

/**********************************  PUBLIC UTILITY FUNCTIONS  *****************************/

/* Called by changelevel() by a timer call. Do not remove! 
   Changes map to specified map by calling changelevel with
   specific parameters. */
public ChangeMap(Timer,Repeat,HLUser,HLParam) {
	new NewMap[MAX_NAME_LENGTH];
	
	convert_string(HLParam,NewMap,MAX_NAME_LENGTH);
	if(strlen(NewMap) != 0) {
		changelevel(NewMap, -1);
	}
}

