/*
 * ===========================================================================
 *
 * $Id: plugin_message.sma,v 1.4 2003/03/26 20:44:33 darope Exp $
 *
 *
 * Copyright (c) 1999-2003 Alfred Reynolds, Florian Zschocke, Magua
 *
 *   This file is part of Admin Mod.
 *
 *   Admin Mod is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Admin Mod is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Admin Mod; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ===========================================================================
 *
 * Comments:
 *
 * This script will show a message to everyone who connects,
 * and will show another message every ten minutes. 
 * It contains now all message functions.
 *
 * Changed by Sir D.
 * - moved all say commands of plugin_base to plugin_message
 * - addded helper function convert_string_all
 * - different messages can be set up in admin_motd.txt, which are displayed randomly in admin_repeat_freq
 * - added function admin_set_timeleft to enable/disable say "timeleft"
 * - added function admin_set_nextmap to enable/disable say "nextmap"
 * - changed accesslevel of admin_messagemode to ACCESS_CHAT
 */

#include <string>
#include <admin>
#include <adminlib>

new AM_VERSION_STRING[] = "2.50.60.rc1";

new MessageMode[MAX_PLAYERS][MAX_DATA_LENGTH];

new g_nextmap;
new g_timeleft;

public plugin_init() {
	plugin_registerinfo("AdminMod Message Plugin","All message commands and auto-messaging",AM_VERSION_STRING);
	plugin_registercmd("admin_chat","admin_chat",ACCESS_CHAT,"admin_chat <msg>: Shows message only to other admins.");
	plugin_registercmd("admin_csay","admin_csay",ACCESS_SAY,"admin_csay [color] <msg>: Shows message in center of screen.");
	plugin_registercmd("admin_dmesg","admin_dmesg",ACCESS_CONFIG,"admin_dmesg <idtype> <userid> <message>: Direct message");
	plugin_registercmd("admin_psay","admin_psay",ACCESS_SAY,"admin_psay <target> <msg>: Sends a private msg to target.");
	plugin_registercmd("admin_say","admin_say",ACCESS_SAY,"admin_say <msg>: Shows a message from you as admin.");
	plugin_registercmd("admin_ssay","admin_ssay",ACCESS_SAY,"admin_ssay <msg>: Shows a message admin without identification.");
	plugin_registercmd("admin_tsay","admin_tsay",ACCESS_SAY,"admin_tsay [color] <msg>: Prints msg on lower left of screen.");
	plugin_registercmd("admin_motd","admin_motd",ACCESS_SAY,"admin_motd: Displays a MOTD style window on the client with a msg of your choice.");
	plugin_registercmd("admin_set_timeleft","settimeleft",ACCESS_CONFIG,"admin_set_timeleft <on/off>: enables/disables say ^"timeleft^" in chat.");
	plugin_registercmd("admin_set_nextmap","setnextmap",ACCESS_CONFIG,"admin_set_nextmap <on/off>: enables/disables say ^"nextmap^" in chat.");
	
	plugin_registercmd("admin_messagemode","admin_messagemode",ACCESS_CHAT,"admin_messagemode <command>: Will treat 'say' as command.");
	plugin_registercmd("admin_nomessagemode","admin_messagemode",ACCESS_CHAT,"admin_nomessagemode: Will treat 'say' as 'say'.");
	plugin_registercmd("say","HandleSay",ACCESS_ALL);
	plugin_registerhelp("say",ACCESS_ALL,"say currentmap: Will respond with the current map's name.");
	
	new iRepeatInterval = getvar("admin_repeat_freq");
	if ( iRepeatInterval >= 15 ) {
		set_timer("say_stuff", iRepeatInterval, 99999);
	}
	
	if(!get_vaultnumdata("timeleft",g_timeleft)){
		set_vaultnumdata("timeleft",1);
		g_timeleft=1;
	}
	
	if(g_timeleft){
		plugin_registerhelp("say",ACCESS_ALL,"say timeleft: Will respond with the time left.");
	}
	
	if(!get_vaultnumdata("nextmap",g_nextmap)){
		set_vaultnumdata("nextmap",1);
		g_nextmap=1;
	}
	
	if(g_nextmap){
		plugin_registerhelp("say",ACCESS_ALL,"say nextmap: Will respond with the next map in the cycle.");
	}
	
	return PLUGIN_CONTINUE;
}

/* admin_chat <msg> */
public admin_chat(HLCommand,HLData,HLUserName,UserIndex) {
	new i;
	new maxplayers = maxplayercount();
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Name[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new User[MAX_NAME_LENGTH];
	
	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);

	strstripquotes(Data);
	snprintf(Text, MAX_TEXT_LENGTH, "(Admin) %s: %s", User, Data);
	for(i=1; i<=maxplayers; i++) {
		strinit(Name);
		if(playerinfo(i,Name,MAX_NAME_LENGTH)==1) {
			if(access(ACCESS_CHAT,Name)!=0) {
				messageex(Name, Text, print_chat);
			}
		}
	}
	log(Text);
	return PLUGIN_HANDLED;
}

/* admin_csay [color] <msg> */
public admin_csay(HLCommand,HLData,HLUserName,UserIndex) {
	new Color[MAX_NAME_LENGTH];
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Message[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);

	strgsep( Data, " ", "^"'", Color, MAX_NAME_LENGTH, Message, MAX_DATA_LENGTH );

	if (streq(Color,"red")==1) {
		centersay(Message,10,250,10,10);
	} else if ( streq(Color, "blue")==1) {
		centersay(Message,10,10,10,250);
	} else if ( streq(Color, "green")==1) {
		centersay(Message,10,10,250,10);
	} else if ( streq(Color, "white")==1) {
		centersay(Message,10,250,250,250);
	} else if ( streq(Color, "yellow")==1) {
		centersay(Message,10,250,250,10);
	} else if ( streq(Color, "purple")==1) {
		centersay(Message,10,250,10,250);
	} else if ( streq(Color, "random")==1) {
		centersay(Message,10,random(100),random(100),random(100));
	} else {
		strstripquotes(Data);
		centersay(Data,10,15,250,10);
	}
	log_command(User,Command,Data);
	return PLUGIN_HANDLED;
}

/* admin_dmesg <idtype> <userid> <message> */
public admin_dmesg( HLCommand, HLData, HLUserName, UserIndex ) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new sUser[MAX_NUMBER_LENGTH];
	new sType[10];
	new iUid;
	new uid:tType;
	new sMessage[MAX_TEXT_LENGTH];
	
	new iNumTokens;

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,sUser);


	iNumTokens = strsep( Data, " ", sType, 10, sUser, MAX_NUMBER_LENGTH, sMessage, MAX_TEXT_LENGTH );
	if ( iNumTokens < 2) {
		directmessage( "Incorrect usage, arguments missing." );
		//printf( "num tokens: %d ^n", iNumTokens );
		return PLUGIN_CONTINUE;
	}	// if

	iUid = strtonum( sUser );
	switch( sType[0] ) {
	case 'i':
		tType = uid_index;
	case 's':
		tType = uid_sessionID;
	case 'w':
		tType = uid_wonID;
	default:
		tType = uid_invalid;
	}	// switch()


	strstripquotes(sMessage);
	directmessage( sMessage, iUid, tType );

	return PLUGIN_CONTINUE;
}// admin_dmesg()

/* admin_motd <msg> */
public admin_motd(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[1550];
	new Msg[1500];
	new Target[MAX_TEXT_LENGTH];
	new TargetName[MAX_NAME_LENGTH];

	new User[MAX_NAME_LENGTH];

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);

	strbreak(Data, Target, Msg, MAX_TEXT_LENGTH);
	if (strlen(Msg) == 0) {
		selfmessage( "Unparsable format: no message found.");
		return PLUGIN_HANDLED;
	} else if (check_user(Target) == 0) {
		selfmessage("Unrecognized player: ");
		selfmessage(Target);
		return PLUGIN_HANDLED;
	}
	get_username(Target,TargetName,MAX_NAME_LENGTH);

	motd( TargetName, Msg );

	log_command(User,Command,Data);
	return PLUGIN_HANDLED;
}

/* admin_psay <target> <msg> */
public admin_psay(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new iMsgToAdmin = 0;
	new Msg[MAX_TEXT_LENGTH];
	new Target[MAX_TEXT_LENGTH];
	new TargetName[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new User[MAX_NAME_LENGTH];

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);
	
	strbreak(Data, Target, Msg, MAX_TEXT_LENGTH);
	if (strlen(Msg) == 0) {
		selfmessage( "Unparsable format: no message found.");
		return PLUGIN_HANDLED;
	} else if (check_user(Target) == 0) {
		if (streq(Target, "admin")==1) {
			iMsgToAdmin = 1;
		} else {
			selfmessage("Unrecognized player: ");
			selfmessage(Target);
			return PLUGIN_HANDLED;
		}
	}
	if (iMsgToAdmin == 1) {
		strcpy(TargetName, "Admin", MAX_NAME_LENGTH);
	} else {
		get_username(Target,TargetName,MAX_NAME_LENGTH);
	}

	snprintf(Text, MAX_TEXT_LENGTH, "(Private Msg From %s): %s", User, Msg);
	if (iMsgToAdmin == 1) {
		log(Text);
	} else {
		messageex(TargetName, Text, print_chat);
	}
	snprintf(Text, MAX_TEXT_LENGTH, "(Private Msg To %s): %s", TargetName, Msg);
	if (UserIndex == 0) {
		log(Text);
	} else {
		messageex(User, Text, print_chat);
	}
	log_command(User,Command,Data);
	return PLUGIN_HANDLED;
}

/* admin_say <msg> */
public admin_say(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new User[MAX_NAME_LENGTH];

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);
	strstripquotes(Data);

	snprintf(Text, MAX_TEXT_LENGTH, "Message from Admin (%s): %s", User, Data);
	say(Text);
	return PLUGIN_HANDLED;
}

/* Silent say: does an admin_say, but without showing the user name; exactly as if you did a
'say' from console, or via rcon. */
/* admin_ssay <msg> */
public admin_ssay(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);
	strstripquotes(Data);
	say(Data);
	log_command(User,Command,Data);
	return PLUGIN_HANDLED;
}

/* admin_tsay [color] <msg> */
public admin_tsay(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	new Color[MAX_NAME_LENGTH];
	new Message[MAX_DATA_LENGTH];

	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);

	strgsep( Data, " ", "^"'", Color, MAX_NAME_LENGTH, Message, MAX_DATA_LENGTH );

	if (streq(Color,"red")==1) {
		typesay(Message,10,250,10,10);
	} else if ( streq(Color, "blue")==1) {
		typesay(Message,10,10,10,250);
	} else if ( streq(Color, "green")==1) {
		typesay(Message,10,10,250,10);
	} else if ( streq(Color, "white")==1) {
		typesay(Message,10,250,250,250);
	} else if ( streq(Color, "yellow")==1) {
		typesay(Message,10,250,250,10);
	} else if ( streq(Color, "purple")==1) {
		typesay(Message,10,250,10,250);
	} else if ( streq(Color, "random")==1) {
		typesay(Message,10,random(100),random(100),random(100));
	} else {
		strstripquotes(Data);
		typesay(Data,10,220,220,220);
	}
	log_command(User,Command,Data);
	return PLUGIN_HANDLED;
}

/* admin_messagemode [<command>] 
   admin_nomessagemode */
public admin_messagemode(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new User[MAX_NAME_LENGTH];
	
	if (UserIndex == 0) {
		selfmessage("This command cannot be used from the console.");
		return PLUGIN_HANDLED;
	}
	
	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);
		
	if (streq(Command,"admin_nomessagemode")==1) {
		strinit(MessageMode[UserIndex]);
		selfmessage("Message mode off. Your 'say' commands will be treated as normal.");
	} else if (strlen(Data)==0) {
		strinit(MessageMode[UserIndex]);
		selfmessage("Message mode off. Your 'say' commands will be treated as normal.");
	} else if (strmatch(Data,"say",3)==1) {
		strinit(MessageMode[UserIndex]);
		selfmessage("Message mode off. Your 'say' commands will be treated as normal.");
	} else {
		strcpy(MessageMode[UserIndex], Data, MAX_DATA_LENGTH);
		snprintf(Text, MAX_TEXT_LENGTH,"Message mode on. Your 'say' commands will be treated as '%s'.", MessageMode[UserIndex]);
		selfmessage(Text);
	}
	return PLUGIN_HANDLED;
}

public HandleSay(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	
	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);

	strstripquotes(Data);
	if (strlen(MessageMode[UserIndex]) != 0 && UserIndex != 0) {
	  if ( strcasestr(MessageMode[UserIndex], "admin_") >= 0 ) { // admin_* command
	    plugin_exec( MessageMode[UserIndex], Data );
	    return PLUGIN_HANDLED;
	  } else {   // HL or MOD command
	    snprintf(Text, MAX_TEXT_LENGTH, "%s %s", MessageMode[UserIndex], Data);
	    execclient(User,Text);
	    return PLUGIN_HANDLED;
	  }
	}
	if (streq(Data, "timeleft")==1 && g_timeleft) {
		SayTimeleft();
	} else if (streq(Data, "version")==1) {
		say("Type 'admin_version' in the console for version information.");
	} else if (streq(Data, "nextmap")==1 && g_nextmap) {
		SayNextMap();
	} else if (streq(Data, "currentmap")==1) {
		SayCurrentMap();
	}

	return PLUGIN_CONTINUE;
}

SayCurrentMap() {
	new Text[MAX_TEXT_LENGTH];
	new CurrentMap[MAX_NAME_LENGTH];
	
	currentmap(CurrentMap,MAX_NAME_LENGTH);
	snprintf(Text, MAX_TEXT_LENGTH, "The current map is: %s", CurrentMap);
	say(Text);
}

public setnextmap(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Text[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	new iSet;
	
	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);
	
	iSet=check_param(Data);
	
	if(iSet){
		set_vaultnumdata("nextmap",iSet);
		g_nextmap=iSet;
	}else{
		set_vaultnumdata("nextmap",0);
		g_nextmap=0;
	}
	snprintf(Text,MAX_TEXT_LENGTH,"[ADMIN] say ^"nextmap^" has been set to %i (0=off/1=on)",iSet);
	log_command(User,Command,Data);
	selfmessage(Text);
	return PLUGIN_HANDLED;
}
	

SayNextMap() {
	new Text[MAX_TEXT_LENGTH];
	new NextMap[MAX_NAME_LENGTH];

	nextmap(NextMap,MAX_NAME_LENGTH);
	snprintf(Text, MAX_TEXT_LENGTH, "The next map will be: %s", NextMap);
	say(Text);
}

public settimeleft(HLCommand,HLData,HLUserName,UserIndex) {
	new Command[MAX_COMMAND_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Text[MAX_DATA_LENGTH];
	new User[MAX_NAME_LENGTH];
	new iSet;
	
	convert_string_all(HLCommand,HLData,HLUserName,Command,Data,User);
	
	iSet=check_param(Data);
	
	if(iSet){
		set_vaultnumdata("timeleft",iSet);
		g_timeleft=iSet;
	}else{
		set_vaultnumdata("timeleft",0);
		g_timeleft=0;
	}
	snprintf(Text,MAX_TEXT_LENGTH,"[ADMIN] say ^"timeleft^" has been set to %i (0=off/1=on)",iSet);
	log_command(User,Command,Data);
	selfmessage(Text);
	return PLUGIN_HANDLED;
}

SayTimeleft() {
	new Text[MAX_TEXT_LENGTH];
	new Seconds = timeleft(0);
	Seconds /= 60;	
	snprintf(Text, MAX_TEXT_LENGTH, "Time remaining on map: %i minutes", Seconds);
	say(Text);
}


public plugin_connect(HLUserName, HLIP, UserIndex) {
	set_timer("say_hello",45,0);
	if (UserIndex >= 1 && UserIndex <= MAX_PLAYERS) {
		strinit(MessageMode[UserIndex]);
	}
	return PLUGIN_CONTINUE;
}

public plugin_disconnect(HLUserName, UserIndex) {
	if (UserIndex >= 1 && UserIndex <= MAX_PLAYERS) {
		strinit(MessageMode[UserIndex]);
	}
	return PLUGIN_CONTINUE;
}

public say_hello(Timer,Repeat,HLName,HLParam) {
	new User[MAX_NAME_LENGTH];
	new Msg[MAX_TEXT_LENGTH];
	
	getstrvar("admin_connect_msg",Msg,MAX_TEXT_LENGTH);
	if ( Msg[0] == '0' && Msg[1] == '^0' ) return;
	convert_string(HLName,User,MAX_NAME_LENGTH);
	messageex(User,Msg,print_center);
}

public say_stuff(Timer,Repeat,HLName,HLParam) {
	new Msg[MAX_TEXT_LENGTH];
	new File[MAX_TEXT_LENGTH]="addons/adminmod/config/admin_motd.txt";
	new iFileSize;
	new iLine;
	
	if(fileexists(File)){
		iFileSize=filesize(File);
		if(iFileSize>0){
			iLine=random(iFileSize);
			readfile(File,Msg,iLine,MAX_TEXT_LENGTH);
			if(strlen(Msg)!=0){
				centersay(Msg,10,0,255,0);
				return;
			}
		}
	}
	
	getstrvar("admin_repeat_msg",Msg,MAX_TEXT_LENGTH);
	if ( strlen(Msg)){
		return;
	}
	centersay(Msg,10,0,255,0);
}
